@@ -22,7 +22,8 @@ fps : scalar
     The number of frames per second to read the data at. Default None (i.e.
     read at the file's own fps). One can use this for files with a
     variable fps, or in cases where imageio is unable to correctly detect
-    the fps.
+    the fps. In case of trouble opening camera streams, it may help to set an
+    explicit fps value matching a framerate supported by the camera.
 loop : bool
     If True, the video will rewind as soon as a frame is requested
     beyond the last frame. Otherwise, IndexError is raised. Default False.
@@ -118,15 +119,16 @@ the number of frames can be estimated from the fps and duration in the meta data
 
 """
 
+import re
 import sys
 import time
 import logging
+import platform
 import threading
 import subprocess as sp
 
 import numpy as np
 
-from .. import formats
 from ..core import Format, image_as_uint
 
 logger = logging.getLogger(__name__)
@@ -187,7 +189,7 @@ class FfmpegFormat(Format):
         # Read from video stream?
         # Note that we could write the _video flag here, but a user might
         # select this format explicitly (and this code is not run)
-        if request.filename in ["<video%i>" % i for i in range(10)]:
+        if re.match(r"<video(\d+)>", request.filename):
             return True
 
         # Read from file that we know?
@@ -222,18 +224,24 @@ class FfmpegFormat(Format):
                     "-i",
                     "dummy",
                 ]
-                # Set `shell=True` in sp.Popen to prevent popup of a command line
-                # window in frozen applications. Note: this would be a security
-                # vulnerability if user-input goes into the cmd.
-                proc = sp.Popen(
-                    cmd, stdin=sp.PIPE, stdout=sp.PIPE, stderr=sp.PIPE, shell=True
+                # Set `shell=True` in sp.run to prevent popup of a command
+                # line window in frozen applications. Note: this would be a
+                # security vulnerability if user-input goes into the cmd.
+                # Note that the ffmpeg process returns with exit code 1 when
+                # using `-list_devices` (or `-list_options`), even if the
+                # command is successful, so we set `check=False` explicitly.
+                completed_process = sp.run(
+                    cmd,
+                    stdout=sp.PIPE,
+                    stderr=sp.PIPE,
+                    encoding="utf-8",
+                    shell=True,
+                    check=False,
                 )
-                proc.stdout.readline()
-                proc.terminate()
-                infos = proc.stderr.read().decode("utf-8", errors="ignore")
+
                 # Return device name at index
                 try:
-                    name = parse_device_names(infos)[index]
+                    name = parse_device_names(completed_process.stderr)[index]
                 except IndexError:
                     raise IndexError("No ffdshow camera at index %i." % index)
                 return "video=%s" % name
@@ -290,15 +298,16 @@ class FfmpegFormat(Format):
             self._arg_input_params += ffmpeg_params or []  # backward compat
             # Write "_video"_arg - indicating webcam support
             self.request._video = None
-            if self.request.filename in ["<video%i>" % i for i in range(10)]:
+            regex_match = re.match(r"<video(\d+)>", self.request.filename)
+            if regex_match:
                 self.request._video = self.request.filename
-            # Specify input framerate?
-            if self.request._video:
+            # Specify input framerate? (only on macOS)
+            if self.request._video and platform.system().lower() == "darwin":
                 if "-framerate" not in str(self._arg_input_params):
-                    self._arg_input_params.extend(["-framerate", str(float(fps or 30))])
+                    self._arg_input_params.extend(["-framerate", str(float(fps or 15))])
             # Get local filename
             if self.request._video:
-                index = int(self.request._video[-2])
+                index = int(regex_match.group(1))
                 self._filename = self._get_cam_inputname(index)
             else:
                 self._filename = self.request.get_local_filename()
@@ -704,13 +713,3 @@ def parse_device_names(ffmpeg_output):
         else:
             device_names2.append(friendly_name)  # duplicate, but not much we can do
     return device_names2
-
-
-# Register. You register an *instance* of a Format class.
-format = FfmpegFormat(
-    "ffmpeg",
-    "Many video formats and cameras (via ffmpeg)",
-    ".mov .avi .mpg .mpeg .mp4 .mkv .webm .wmv",
-    "I",
-)
-formats.add_format(format)
