@@ -3,6 +3,7 @@
 
 import os
 import sys
+import io
 from zipfile import ZipFile
 import numpy as np
 
@@ -213,7 +214,8 @@ def test_jpg_more():
     open(fname, "wb").write(b"this is not an image")
     raises(Exception, imageio.imread, fname)
     #
-    bb = imageio.imsave(imageio.RETURN_BYTES, get_ref_im(3, 0, 0), "JPEG")
+    img = get_ref_im(3, 0, 0)
+    bb = imageio.imsave(imageio.RETURN_BYTES, img, "JPEG-PIL")
     with open(fname, "wb") as f:
         f.write(bb[:400])
         f.write(b" ")
@@ -419,6 +421,19 @@ def test_scipy_imread_compat():
     im = imageio.imread("imageio:chelsea.png", "PNG-PIL")
 
 
+def test_write_jpg_to_bytes_io():
+    # this is a regression test
+    # see: https://github.com/imageio/imageio/issues/687
+
+    image = np.zeros((200, 200), dtype=np.uint8)
+    bytes_io = io.BytesIO()
+    imageio.imwrite(bytes_io, image, "jpeg")
+    bytes_io.seek(0)
+
+    image_from_file = imageio.imread(bytes_io)
+    assert np.allclose(image_from_file, image)
+
+
 if __name__ == "__main__":
     # test_inside_zipfile()
     # test_png()
